// Copyright (C) 2023, Jakob Wakeling
// All rights reserved.

package repo

import (
	"regexp"
	"strings"

	"github.com/Jamozed/Goit/src/goit"
	"github.com/Jamozed/Goit/src/util"
	"github.com/go-git/go-git/v5"
	"github.com/go-git/go-git/v5/plumbing"
)

type HeaderFields struct {
	Name, Description, URL, SSH string
	Readme, Licence, Mirror     string
	Editable, EnableSSH         bool
}

func GetHeaderFields(auth bool, user *goit.User, repo *goit.Repo, host string) HeaderFields {
	var hostWithoutPort = host
	if strings.LastIndex(host, ":") != -1 {
		hostWithoutPort = host[:strings.LastIndex(host, ":")]
	}

	return HeaderFields{
		Name: repo.Name, Description: repo.Description,
		URL:       util.If(goit.Conf.UsesHTTPS, "https://", "http://") + host + "/" + repo.Name,
		Editable:  (auth && repo.OwnerId == user.Id),
		Mirror:    util.If(repo.IsMirror, repo.Upstream, ""),
		EnableSSH: goit.Conf.EnableSSH,
		SSH:       goit.Conf.SSHUsername + "@" + hostWithoutPort + ":" + repo.Name,
	}
}

var readmePattern = regexp.MustCompile(`(?i)^readme(?:\.?(?:md|txt))?$`)
var licencePattern = regexp.MustCompile(`(?i)^licence(?:\.?(?:md|txt))?$`)

/* Find a file that matches a regular expression in the root level of a reference. */
func findPattern(gr *git.Repository, ref *plumbing.Reference, re *regexp.Regexp) (string, error) {
	c, err := gr.CommitObject(ref.Hash())
	if err != nil {
		return "", err
	}

	t, err := c.Tree()
	if err != nil {
		return "", err
	}

	for _, e := range t.Entries {
		if re.MatchString(e.Name) {
			return e.Name, nil
		}
	}

	return "", nil
}
